@@ -32,6 +32,8 @@ class KigDocument;
 class CoordinateSystem;
 
 class KigCommandTask;
+class KigWidget;
+class Rect;
 
 class KigCommand
   : public QObject, public KNamedCommand
@@ -43,6 +45,10 @@ public:
   KigCommand( KigDocument& inDoc, const QString& name );
   ~KigCommand();
 
+  /**
+   * To avoid confusion, this doesn't add a command to anything, this
+   * creates an AddCommand ;)
+   */
   static KigCommand* addCommand( KigDocument& doc, const Objects& os );
   static KigCommand* addCommand( KigDocument& doc, Object* os );
   /**
@@ -81,7 +87,10 @@ public:
   void unexecute( KigDocument& doc );
 protected:
   bool undone;
-  Objects os;
+
+  // we keep a reference to the objects we contain, so they don't get
+  // deleted..
+  ReferenceObject mobjsref;
 };
 
 class RemoveObjectsTask
@@ -176,4 +185,17 @@ public:
   void unexecute( KigDocument& doc );
 };
 
+class KigViewShownRectChangeTask
+  : public KigCommandTask
+{
+  class Private;
+  Private* d;
+public:
+  KigViewShownRectChangeTask( KigWidget& v, const Rect& newrect );
+  ~KigViewShownRectChangeTask();
+
+  void execute( KigDocument& doc );
+  void unexecute( KigDocument& doc );
+};
+
 #endif
