@@ -22,9 +22,10 @@
 #include "kig_view.moc"
 
 #include "kig_part.h"
+#include "kig_commands.h"
 #include "../objects/object.h"
 #include "../misc/coordinate_system.h"
-
+#include "../misc/kigpainter.h"
 #include "../modes/mode.h"
 #include "../modes/dragrectmode.h"
 
@@ -47,14 +48,15 @@ kdbgstream& operator<< ( kdbgstream& s, const QPoint& t )
 {
   s << "x: " << t.x() << " y: " << t.y();
   return s;
-};
+}
 
 KigWidget::KigWidget( KigDocument* doc,
                       KigView* view,
                       QWidget* parent,
                       const char* name,
                       bool fullscreen )
-  : QWidget(parent, name, fullscreen ? WStyle_Customize | WStyle_NoBorder : 0 ),
+  : QWidget( parent, name,
+             fullscreen ? WStyle_Customize | WStyle_NoBorder : 0 ),
     mdocument( doc ),
     mview( view ),
     stillPix(size()),
@@ -70,12 +72,12 @@ KigWidget::KigWidget( KigDocument* doc,
 
   curPix.resize( size() );
   stillPix.resize( size() );
-};
+}
 
 KigWidget::~KigWidget()
 {
   mdocument->delWidget( this );
-};
+}
 
 void KigWidget::paintEvent(QPaintEvent*)
 {
@@ -90,7 +92,7 @@ void KigWidget::mousePressEvent (QMouseEvent* e)
     return mdocument->mode()->midClicked( e, this );
   if ( e->button() & Qt::RightButton )
     return mdocument->mode()->rightClicked( e, this );
-};
+}
 
 void KigWidget::mouseMoveEvent (QMouseEvent* e)
 {
@@ -101,7 +103,7 @@ void KigWidget::mouseMoveEvent (QMouseEvent* e)
   if ( e->state() & Qt::RightButton )
     return mdocument->mode()->rightMouseMoved( e, this );
   return mdocument->mode()->mouseMoved( e, this );
-};
+}
 
 void KigWidget::mouseReleaseEvent (QMouseEvent* e)
 {
@@ -111,7 +113,7 @@ void KigWidget::mouseReleaseEvent (QMouseEvent* e)
     return mdocument->mode()->midReleased( e, this );
   if ( e->state() & Qt::RightButton )
     return mdocument->mode()->rightReleased( e, this );
-};
+}
 
 void KigWidget::updateWidget( const std::vector<QRect>& overlay )
 {
@@ -132,7 +134,7 @@ void KigWidget::updateWidget( const std::vector<QRect>& overlay )
 #endif
   };
   oldOverlay = overlay;
-};
+}
 
 void KigWidget::updateEntireWidget()
 {
@@ -164,9 +166,8 @@ void KigWidget::resizeEvent( QResizeEvent* e )
   // don't..
   if ( nsize.width() / osize.width() > 4 ) recenterScreen();
 
-  updateScrollBars();
-
   redrawScreen();
+  updateScrollBars();
 }
 
 void KigWidget::updateCurPix( const std::vector<QRect>& ol )
@@ -185,8 +186,6 @@ void KigWidget::updateCurPix( const std::vector<QRect>& ol )
 void KigWidget::recenterScreen()
 {
   msi.setShownRect( matchScreenShape( mdocument->suggestedRect() ) );
-  updateScrollBars();
-  redrawScreen();
 }
 
 Rect KigWidget::matchScreenShape( const Rect& r ) const
@@ -194,26 +193,37 @@ Rect KigWidget::matchScreenShape( const Rect& r ) const
   return r.matchShape( Rect::fromQRect( rect() ) );
 }
 
-void KigWidget::zoomIn()
+void KigWidget::slotZoomIn()
 {
-  Rect r = msi.shownRect();
-  Coordinate c = r.center();
-  r /= 2;
-  r.setCenter( c );
-  msi.setShownRect( r );
-  redrawScreen();
-  updateScrollBars();
+  Rect nr = msi.shownRect();
+  Coordinate c = nr.center();
+  nr /= 2;
+  nr.setCenter( c );
+  KigCommand* cd =
+    new KigCommand( *mdocument,
+                    i18n( "Zoom In" ) );
+  cd->addTask( new KigViewShownRectChangeTask( *this, nr ) );
+  mdocument->history()->addCommand( cd );
 }
 
-void KigWidget::zoomOut()
+void KigWidget::slotZoomOut()
 {
-  Rect r = msi.shownRect();
-  Coordinate c = r.center();
-  r *= 2;
-  r.setCenter( c );
-  msi.setShownRect( r );
-  redrawScreen();
-  updateScrollBars();
+  Rect nr = msi.shownRect();
+  Coordinate c = nr.center();
+  nr *= 2;
+  nr.setCenter( c );
+
+  // zooming in is undoable..  I know this isn't really correct,
+  // because the current view doesn't really belong to the document (
+  // althought KGeo and KSeg both save them along, iirc ).  However,
+  // undoing a zoom or another operation affecting the window seems a
+  // bit too useful to not be available.  Please try to convince me if
+  // you feel otherwise ;-)
+  KigCommand* cd =
+    new KigCommand( *mdocument,
+                    i18n( "Zoom Out" ) );
+  cd->addTask( new KigViewShownRectChangeTask( *this, nr ) );
+  mdocument->history()->addCommand( cd );
 }
 
 void KigWidget::clearStillPix()
@@ -274,7 +284,7 @@ KigView::KigView( KigDocument* doc,
     mupdatingscrollbars( false ),
     mrealwidget( 0 ), mdoc( doc )
 {
-  connect( doc, SIGNAL( recenterScreen() ), this, SLOT( recenterScreen() ) );
+  connect( doc, SIGNAL( recenterScreen() ), this, SLOT( slotRecenterScreen() ) );
 
   mlayout = new QGridLayout( this, 2, 2 );
   mrightscroll = new QScrollBar( Vertical, this, "Right Scrollbar" );
@@ -296,6 +306,8 @@ KigView::KigView( KigDocument* doc,
 
   resize( sizeHint() );
   mrealwidget->recenterScreen();
+  mrealwidget->redrawScreen();
+  updateScrollBars();
 }
 
 void KigView::updateScrollBars()
@@ -411,7 +423,7 @@ KigView::~KigView()
 {
 }
 
-KigWidget* KigView::realWidget()
+KigWidget* KigView::realWidget() const
 {
   return mrealwidget;
 }
@@ -423,7 +435,7 @@ const KigDocument& KigWidget::document() const
 
 QSize KigWidget::sizeHint() const
 {
-  return QSize( 600, 400 );
+  return QSize( 630, 450 );
 }
 
 void KigWidget::wheelEvent( QWheelEvent* e )
@@ -447,19 +459,25 @@ bool KigWidget::isFullScreen() const
   return misfullscreen;
 }
 
-void KigView::zoomIn()
+void KigView::slotZoomIn()
 {
-  mrealwidget->zoomIn();
+  mrealwidget->slotZoomIn();
 }
 
-void KigView::zoomOut()
+void KigView::slotZoomOut()
 {
-  mrealwidget->zoomOut();
+  mrealwidget->slotZoomOut();
 }
 
-void KigView::recenterScreen()
+void KigWidget::slotRecenterScreen()
 {
-  mrealwidget->recenterScreen();
+  Rect nr = mdocument->suggestedRect();
+  KigCommand* cd =
+    new KigCommand( *mdocument,
+                    i18n( "Recenter View" ) );
+
+  cd->addTask( new KigViewShownRectChangeTask( *this, nr ) );
+  mdocument->history()->addCommand( cd );
 }
 
 void KigView::toggleFullScreen()
@@ -481,10 +499,17 @@ void KigWidget::zoomRect()
   mdocument->emitStatusBarText( i18n( "Select the rectangle that should be shown." ) );
   DragRectMode d( *mdocument, *this );
   mdocument->runMode( &d );
-  Rect nr = d.rect();
+  if ( ! d.cancelled() )
+  {
+    Rect nr = d.rect();
+    KigCommand* cd =
+      new KigCommand( *mdocument,
+                      i18n( "Change Shown Part of Screen" ) );
+
+    cd->addTask( new KigViewShownRectChangeTask( *this, nr ) );
+    mdocument->history()->addCommand( cd );
+  };
 
-  nr = nr.matchShape( Rect::fromQRect( rect() ) );
-  msi.setShownRect( nr );
   redrawScreen();
   updateScrollBars();
 }
@@ -493,3 +518,13 @@ void KigView::zoomRect()
 {
   mrealwidget->zoomRect();
 }
+
+void KigWidget::setShowingRect( const Rect& r )
+{
+  msi.setShownRect( r.matchShape( Rect::fromQRect( rect() ) ) );
+}
+
+void KigView::slotRecenterScreen()
+{
+  mrealwidget->slotRecenterScreen();
+}
