@@ -23,6 +23,8 @@
 
 #include "../objects/common.h"
 
+class ObjectImpType;
+
 class ArgsChecker
 {
 public:
@@ -43,7 +45,7 @@ class ArgParser
   : public ArgsChecker
 {
 public:
-  struct spec { int type; const char* usetext; };
+  struct spec { const ObjectImpType* type; const char* usetext; };
 private:
   // the args spec..
   std::vector<spec> margs;
@@ -57,15 +59,19 @@ public:
   ArgParser( const struct spec* args, int n );
   ArgParser( const std::vector<spec>& args );
   ArgParser( const std::vector<spec>& args, const std::vector<const char*> anyobjsspec );
+  ArgParser();
+
+  void initialize( const std::vector<spec>& args );
+  void initialize( const struct spec* args, int n );
   // returns a new ArgParser that wants the same args, except for the
   // ones of the given type..
-  ArgParser without( int type ) const;
+  ArgParser without( const ObjectImpType* type ) const;
   // checks if os matches the argument list this parser should parse..
   int check( const Objects& os ) const;
   int check( const Args& os ) const;
   // returns the usetext for the argument that o would be used for,
-  // if sel.with( o ) were used as parents..
-  // o is not in sel.
+  // if sel were used as parents..
+  // o should be in sel...
   const char* usetext( const ObjectImp* o, const Args& sel ) const;
 
   // this reorders the objects or args so that they are in the same
@@ -73,7 +79,9 @@ public:
   Objects parse( const Objects& os ) const;
   Args parse( const Args& os ) const;
 
-  int impRequirement( const ObjectImp* o, const Args& parents ) const;
+  // returns the minimal ObjectImp ID that o needs to inherit in order
+  // to be useful..  o should be part of parents.
+  const ObjectImpType* impRequirement( const ObjectImp* o, const Args& parents ) const;
 };
 
 #endif
