@@ -18,6 +18,8 @@
 
 #include "builtin_stuff.h"
 
+#include <config.h>
+
 #include "object_constructor.h"
 #include "lists.h"
 #include "special_constructors.h"
@@ -33,6 +35,8 @@
 #include "../objects/other_type.h"
 #include "../objects/transform_types.h"
 #include "../objects/point_type.h"
+#include "../objects/other_imp.h"
+#include "../objects/line_imp.h"
 
 void setupBuiltinStuff()
 {
@@ -49,14 +53,14 @@ void setupBuiltinStuff()
       I18N_NOOP( "A segment constructed from its start and end point" ),
       "segment" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_segment" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_segment", Qt::Key_S ) );
 
     // line by two points..
     c = new SimpleObjectTypeConstructor(
       LineABType::instance(), I18N_NOOP( "Line by Two Points" ),
       I18N_NOOP( "A line constructed through two points"), "line" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_linettp" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_linettp", Qt::Key_L ) );
 
     // ray by two points..
     c = new SimpleObjectTypeConstructor(
@@ -64,7 +68,7 @@ void setupBuiltinStuff()
       I18N_NOOP( "A ray by its start point, and another point somewhere on it." ),
       "ray" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_ray" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_ray", Qt::Key_R ) );
 
     // perpendicular line
     c = new SimpleObjectTypeConstructor(
@@ -88,7 +92,7 @@ void setupBuiltinStuff()
       I18N_NOOP( "A circle constructed by its center and a point on its border" ),
       "circlebcp" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_circlebcp" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_circlebcp", Qt::Key_C ) );
 
     c = new SimpleObjectTypeConstructor(
       CircleBTPType::instance(), I18N_NOOP( "Circle by Three Points" ),
@@ -106,7 +110,7 @@ void setupBuiltinStuff()
     // we only register the guiaction, cause it makes sense to have a
     // toolbar icon for this..
     static PropertyObjectConstructor anglebisectionctor(
-      ObjectImp::ID_AngleImp, I18N_NOOP( "Construct the bisector of this angle" ),
+      AngleImp::stype(), I18N_NOOP( "Construct the bisector of this angle" ),
       I18N_NOOP( "Angle Bisector" ), I18N_NOOP( "The bisector of an angle" ),
       "angle-bisector", "angle-bisector" );
     actions->add( new ConstructibleAction( &anglebisectionctor, "objects_new_angle_bisector" ) );
@@ -213,7 +217,7 @@ void setupBuiltinStuff()
       I18N_NOOP( "An angle defined by three points" ),
       "angle" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_angle" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_angle", Qt::Key_A ) );
 
     c = new SimpleObjectTypeConstructor(
       EquilateralHyperbolaB4PType::instance(),
@@ -224,18 +228,29 @@ void setupBuiltinStuff()
     actions->add( new ConstructibleAction( c, "objects_new_equilateralhyperbolab4p" ) );
 
     {
+      // now for the Mid Point action.  It does both the mid point of
+      // a segment, and the mid point of two points.  The midpoint of
+      // two segments just shows the mid point property, and therefore
+      // doesn't need to be added to the ctors, because there are
+      // already facilities to construct an object's properties..
+      // therefore, we add only an mpotp to the ctors, and add the
+      // merged constructor only to the actions..
+      ctors->add( new MidPointOfTwoPointsConstructor() );
+
       ObjectConstructor* mpotp = new MidPointOfTwoPointsConstructor();
       ObjectConstructor* mpos = new PropertyObjectConstructor(
-        ObjectImp::ID_SegmentImp, I18N_NOOP( "Construct the midpoint of this segment" ),
+        SegmentImp::stype(), I18N_NOOP( "Construct the midpoint of this segment" ),
         "", "", "", "mid-point" );
-      MergeObjectConstructor* m = new MergeObjectConstructor(
-        I18N_NOOP( "Midpoint" ),
+
+      // make this a static object, so it gets deleted at the end of
+      // the program.
+      static MergeObjectConstructor m(
+        I18N_NOOP( "Mid Point" ),
         I18N_NOOP( "The midpoint of a segment or two other points" ),
         "bisection" );
-      m->merge( mpotp );
-      m->merge( mpos );
-      ctors->add( m );
-      actions->add( new ConstructibleAction( m, "objects_new_midpoint" ) );
+      m.merge( mpotp );
+      m.merge( mpos );
+      actions->add( new ConstructibleAction( &m, "objects_new_midpoint", Qt::Key_M ) );
     };
 
     c = new SimpleObjectTypeConstructor(
@@ -244,7 +259,7 @@ void setupBuiltinStuff()
       I18N_NOOP( "Construct a vector from two given points." ),
       "vector" );
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_vector" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_vector", Qt::Key_V ) );
 
     c = new SimpleObjectTypeConstructor(
       ArcBTPType::instance(),
@@ -346,11 +361,20 @@ void setupBuiltinStuff()
     // the generic intersection constructor..
     c = new GenericIntersectionConstructor();
     ctors->add( c );
-    actions->add( new ConstructibleAction( c, "objects_new_intersection" ) );
+    actions->add( new ConstructibleAction( c, "objects_new_intersection", Qt::Key_I ) );
 
     actions->add( new ConstructPointAction( "objects_new_normalpoint" ) );
     actions->add( new ConstructTextLabelAction( "objects_new_textlabel" ) );
+    actions->add( new AddFixedPointAction( "objects_new_point_xy" ) );
+
+#ifdef KIG_ENABLE_PYTHON_SCRIPTING
+    actions->add( new NewScriptAction( "objects_new_script_object" ) );
+#endif
+
+#if 0
+    actions->add( new TestAction( "test_stuff" ) );
+#endif
   };
 
   done = true;
-};
+}
