@@ -27,13 +27,13 @@
 CubicCartesianData::CubicCartesianData()
 {
   std::fill( coeffs, coeffs + 10, 0 );
-};
+}
 
 CubicCartesianData::CubicCartesianData(
             const double incoeffs[10] )
 {
   std::copy( incoeffs, incoeffs + 10, coeffs );
-};
+}
 
 const CubicCartesianData calcCubicThroughPoints (
     const std::vector<Coordinate>& points )
@@ -121,7 +121,8 @@ const CubicCartesianData calcCubicThroughPoints (
     if (addedconstraint) ++numpoints;
   }
 
-  GaussianElimination( matrix, numpoints, 10, scambio );
+  if ( ! GaussianElimination( matrix, numpoints, 10, scambio ) )
+    return CubicCartesianData::invalidData();
   // fine della fase di eliminazione
   BackwardSubstitution( matrix, numpoints, 10, scambio, solution );
 
@@ -214,7 +215,8 @@ const CubicCartesianData calcCubicCuspThroughPoints (
     if (addedconstraint) ++numpoints;
   }
 
-  GaussianElimination( matrix, numpoints, 10, scambio );
+  if ( ! GaussianElimination( matrix, numpoints, 10, scambio ) )
+    return CubicCartesianData::invalidData();
   // fine della fase di eliminazione
   BackwardSubstitution( matrix, numpoints, 10, scambio, solution );
 
@@ -307,7 +309,8 @@ const CubicCartesianData calcCubicNodeThroughPoints (
     if (addedconstraint) ++numpoints;
   }
 
-  GaussianElimination( matrix, numpoints, 10, scambio );
+  if ( ! GaussianElimination( matrix, numpoints, 10, scambio ) )
+    return CubicCartesianData::invalidData();
   // fine della fase di eliminazione
   BackwardSubstitution( matrix, numpoints, 10, scambio, solution );
 
@@ -504,4 +507,16 @@ bool operator==( const CubicCartesianData& lhs, const CubicCartesianData& rhs )
     if ( lhs.coeffs[i] != rhs.coeffs[i] )
       return false;
   return true;
-};
+}
+
+CubicCartesianData CubicCartesianData::invalidData()
+{
+  CubicCartesianData ret;
+  ret.coeffs[0] = double_inf;
+  return ret;
+}
+
+bool CubicCartesianData::valid() const
+{
+  return finite( coeffs[0] );
+}
