@@ -19,6 +19,8 @@
 #ifndef KIG_MISC_GUIACTION_H
 #define KIG_MISC_GUIACTION_H
 
+#include <config.h>
+
 #include <qstring.h>
 #include <qcstring.h>
 #include <kaction.h>
@@ -52,7 +54,7 @@ public:
   virtual QCString iconFileName() const = 0;
   virtual QString descriptiveName() const = 0;
   virtual const char* actionName() const = 0;
-  virtual int shortcut() const;
+  virtual int shortcut() const = 0;
   virtual void act( KigDocument& ) = 0;
 
   virtual void plug( KigDocument* doc, KigGUIAction* kact );
@@ -65,13 +67,16 @@ class ConstructibleAction
 {
   ObjectConstructor* mctor;
   QCString mactionname;
+  int mshortcut;
 public:
-  ConstructibleAction( ObjectConstructor* ctor, const QCString& actionname );
+  ConstructibleAction( ObjectConstructor* ctor, const QCString& actionname,
+                       int shortcut = 0 );
   ~ConstructibleAction();
   QString description() const;
   QCString iconFileName() const;
   QString descriptiveName() const;
   const char* actionName() const;
+  int shortcut() const;
   void act( KigDocument& );
   void plug( KigDocument* doc, KigGUIAction* kact );
 };
@@ -103,6 +108,7 @@ public:
   QCString iconFileName() const;
   QString descriptiveName() const;
   const char* actionName() const;
+  int shortcut() const;
   void act( KigDocument& );
 };
 
@@ -117,9 +123,11 @@ public:
   QCString iconFileName() const;
   QString descriptiveName() const;
   const char* actionName() const;
+  int shortcut() const;
   void act( KigDocument& );
 };
 
+#if 0
 class TestAction
   : public GUIAction
 {
@@ -133,5 +141,25 @@ public:
   const char* actionName() const;
   void act( KigDocument& );
 };
+#endif
+
+#ifdef KIG_ENABLE_PYTHON_SCRIPTING
+
+class NewScriptAction
+  : public GUIAction
+{
+  const char* mactionname;
+public:
+  NewScriptAction( const char* actionname );
+  ~NewScriptAction();
+  QString description() const;
+  QCString iconFileName() const;
+  QString descriptiveName() const;
+  const char* actionName() const;
+  void act( KigDocument& );
+  int shortcut() const;
+};
+
+#endif // KIG_ENABLE_PYTHON_SCRIPTING
 
 #endif
