@@ -47,7 +47,7 @@ public:
 
 ObjectHierarchy::Node::~Node()
 {
-};
+}
 
 class PushStackNode
   : public ObjectHierarchy::Node
@@ -69,25 +69,25 @@ public:
 void PushStackNode::apply( Objects& stack, int loc ) const
 {
   stack[loc] = new DataObject( mimp->copy() );
-};
+}
 
-int PushStackNode::id() const { return ID_PushStack; };
+int PushStackNode::id() const { return ID_PushStack; }
 
 PushStackNode::~PushStackNode()
 {
   delete mimp;
-};
+}
 
 ObjectHierarchy::Node* PushStackNode::copy() const
 {
   return new PushStackNode( mimp->copy() );
-};
+}
 
 void PushStackNode::apply( std::vector<const ObjectImp*>& stack,
                            int loc, const KigDocument& ) const
 {
   stack[loc] = mimp->copy();
-};
+}
 
 class ApplyTypeNode
   : public ObjectHierarchy::Node
@@ -109,7 +109,7 @@ public:
   void apply( Objects& stack, int loc ) const;
 };
 
-int ApplyTypeNode::id() const { return ID_ApplyType; };
+int ApplyTypeNode::id() const { return ID_ApplyType; }
 
 ApplyTypeNode::~ApplyTypeNode()
 {
@@ -118,7 +118,7 @@ ApplyTypeNode::~ApplyTypeNode()
 ObjectHierarchy::Node* ApplyTypeNode::copy() const
 {
   return new ApplyTypeNode( mtype, mparents );
-};
+}
 
 void ApplyTypeNode::apply( Objects& stack, int loc ) const
 {
@@ -126,7 +126,7 @@ void ApplyTypeNode::apply( Objects& stack, int loc ) const
   for ( uint i = 0; i < mparents.size(); ++i )
     parents.push_back( stack[ mparents[i] ] );
   stack[loc] = new RealObject( mtype, parents );
-};
+}
 
 void ApplyTypeNode::apply( std::vector<const ObjectImp*>& stack,
                            int loc, const KigDocument& doc ) const
@@ -137,7 +137,7 @@ void ApplyTypeNode::apply( std::vector<const ObjectImp*>& stack,
     args.push_back( stack[mparents[i]] );
   };
   stack[loc] = mtype->calc( args, doc );
-};
+}
 
 class FetchPropertyNode
   : public ObjectHierarchy::Node
@@ -166,17 +166,17 @@ public:
 
 FetchPropertyNode::~FetchPropertyNode()
 {
-};
+}
 
 ObjectHierarchy::Node* FetchPropertyNode::copy() const
 {
   return new FetchPropertyNode( mparent, mname, mpropid );
-};
+}
 
 int FetchPropertyNode::id() const
 {
   return ID_FetchProp;
-};
+}
 
 void FetchPropertyNode::apply( std::vector<const ObjectImp*>& stack,
                                int loc, const KigDocument& d ) const
@@ -185,14 +185,14 @@ void FetchPropertyNode::apply( std::vector<const ObjectImp*>& stack,
   if ( mpropid == -1 ) mpropid = stack[mparent]->propertiesInternalNames().findIndex( mname );
   assert( mpropid != -1 );
   stack[loc] = stack[mparent]->property( mpropid, d );
-};
+}
 
 void FetchPropertyNode::apply( Objects& stack, int loc ) const
 {
   if ( mpropid == -1 ) mpropid = stack[mparent]->propertiesInternalNames().findIndex( mname );
   assert( mpropid != -1 );
   stack[loc] = new PropertyObject( stack[mparent], mpropid );
-};
+}
 
 std::vector<ObjectImp*> ObjectHierarchy::calc( const Args& a, const KigDocument& doc ) const
 {
@@ -263,6 +263,7 @@ int ObjectHierarchy::visit( const Object* o, std::map<const Object*, int>& seenm
   {
     if ( isresult )
     {
+      assert( ! o->imp()->isCache() );
       // o is a result object that does not descend from the given
       // objects..  We have to just save its value here, I guess..
       mnodes.push_back( new PushStackNode( o->imp()->copy() ) );
@@ -272,38 +273,7 @@ int ObjectHierarchy::visit( const Object* o, std::map<const Object*, int>& seenm
       return -1;
   };
 
-  for ( uint i = 0; i < p.size(); ++i )
-  {
-    if ( parents[i] == -1 )
-    {
-      mnodes.push_back( new PushStackNode( p[i]->imp()->copy() ) );
-      parents[i] = mnumberofargs + mnodes.size() - 1;
-    }
-    else if ( (uint) parents[i] < mnumberofargs )
-    {
-      Object* parent = o->parents()[i];
-      Objects oparents = o->parents();
-
-      margrequirements[parents[i]] = kMax( margrequirements[parents[i]],
-                                           o->impRequirement( parent, oparents ) );
-    };
-  };
-  if ( o->inherits( Object::ID_RealObject ) )
-    mnodes.push_back( new ApplyTypeNode( static_cast<const RealObject*>( o )->type(), parents ) );
-  else if ( o->inherits( Object::ID_PropertyObject ) )
-  {
-    assert( parents.size() == 1 );
-    int parent = parents.front();
-    Object* op = p.front();
-    assert( op );
-    uint propid = static_cast<const PropertyObject*>( o )->propId();
-    assert( propid < op->propertiesInternalNames().size() );
-    mnodes.push_back( new FetchPropertyNode( parent, op->propertiesInternalNames()[propid], propid ) );
-  }
-  else
-    assert( false );
-  seenmap[o] = mnumberofargs + mnodes.size() - 1;
-  return mnumberofargs + mnodes.size() - 1;
+  return storeObject( o, p, parents, seenmap );
 }
 
 ObjectHierarchy::~ObjectHierarchy()
@@ -331,7 +301,10 @@ ObjectHierarchy ObjectHierarchy::withFixedArgs( const Args& a ) const
   std::vector<Node*> newnodes( mnodes.size() + a.size() );
   std::vector<Node*>::iterator newnodesiter = newnodes.begin();
   for ( uint i = 0; i < a.size(); ++i )
+  {
+    assert( ! a[i]->isCache() );
     *newnodesiter++ = new PushStackNode( a[i]->copy() );
+  };
   std::copy( ret.mnodes.begin(), ret.mnodes.end(), newnodesiter );
   ret.mnodes = newnodes;
 
@@ -341,7 +314,7 @@ ObjectHierarchy ObjectHierarchy::withFixedArgs( const Args& a ) const
 ObjectHierarchy::ObjectHierarchy( const Objects& from, const Object* to )
   : mnumberofargs( from.size() ), mnumberofresults( 1 )
 {
-  margrequirements.resize( from.size(), ObjectImp::ID_AnyImp );
+  margrequirements.resize( from.size(), ObjectImp::stype() );
   std::map<const Object*, int> seenmap;
   for ( uint i = 0; i < from.size(); ++i )
     seenmap[from[i]] = i;
@@ -355,7 +328,7 @@ ObjectHierarchy::ObjectHierarchy( const Objects& from, const Object* to )
 ObjectHierarchy::ObjectHierarchy( const Objects& from, const Objects& to )
   : mnumberofargs( from.size() ), mnumberofresults( to.size() )
 {
-  margrequirements.resize( from.size(), ObjectImp::ID_AnyImp );
+  margrequirements.resize( from.size(), ObjectImp::stype() );
   std::map<const Object*, int> seenmap;
   for ( uint i = 0; i < from.size(); ++i )
     seenmap[from[i]] = i;
@@ -377,7 +350,7 @@ void ObjectHierarchy::serialize( QDomElement& parent, QDomDocument& doc ) const
   {
     QDomElement e = doc.createElement( "input" );
     e.setAttribute( "id", id++ );
-    e.setAttribute( "requirement", ObjectImp::idToString( margrequirements[i] ) );
+    e.setAttribute( "requirement", margrequirements[i]->internalName() );
     parent.appendChild( e );
   }
 
@@ -439,9 +412,9 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
     mnumberofargs = kMax( id, mnumberofargs );
 
     tmp = e.attribute( "requirement" );
-    int req = ObjectImp::stringToID( tmp.latin1() );
-    if ( req == -1 ) req = ObjectImp::ID_AnyImp; // sucks, i know..
-    margrequirements.resize( mnumberofargs, ObjectImp::ID_AnyImp );
+    const ObjectImpType* req = ObjectImpType::typeFromInternalName( tmp.latin1() );
+    if ( req == 0 ) req = ObjectImp::stype(); // sucks, i know..
+    margrequirements.resize( mnumberofargs, ObjectImp::stype() );
     margrequirements[id - 1] = req;
   }
   for (; !e.isNull(); e = e.nextSibling().toElement() )
@@ -501,10 +474,10 @@ ArgParser ObjectHierarchy::argParser() const
   std::vector<ArgParser::spec> specs;
   for ( uint i = 0; i < margrequirements.size(); ++i )
   {
-    int req = margrequirements[i];
+    const ObjectImpType* req = margrequirements[i];
     ArgParser::spec spec;
     spec.type = req;
-    spec.usetext = ObjectImp::selectStatement( req );
+    spec.usetext = req->selectStatement();
     assert( spec.usetext );
     specs.push_back( spec );
   };
@@ -530,18 +503,18 @@ Objects ObjectHierarchy::buildObjects( const Objects& os, const KigDocument& doc
   for ( uint i = mnumberofargs; i < stack.size() - mnumberofresults; ++i )
     stack[i]->setShown( false );
 
-  Objects ret( stack.begin() + mnumberofargs, stack.end() );
+  Objects ret( stack.end() - mnumberofresults, stack.end() );
 
   return ret;
 }
 
-int ObjectHierarchy::idOfLastResult() const
+const ObjectImpType* ObjectHierarchy::idOfLastResult() const
 {
   const Node* n = mnodes.back();
   if ( n->id() == Node::ID_PushStack )
-    return static_cast<const PushStackNode*>( n )->imp()->id();
+    return static_cast<const PushStackNode*>( n )->imp()->type();
   else if ( n->id() == Node::ID_FetchProp )
-    return ObjectImp::ID_AnyImp;
+    return ObjectImp::stype();
   else
     return static_cast<const ApplyTypeNode*>( n )->type()->resultId();
 }
@@ -575,7 +548,7 @@ bool operator==( const ObjectHierarchy& lhs, const ObjectHierarchy& rhs )
       return false;
 
   return true;
-};
+}
 
 bool ObjectHierarchy::resultDoesNotDependOnGiven() const
 {
@@ -586,3 +559,71 @@ bool ObjectHierarchy::resultDoesNotDependOnGiven() const
   };
   return false;
 }
+
+// returns the "minimum" of a and b ( in the partially ordered set of
+// ObjectImpType's, using the inherits member function as comparison,
+// if you for some reason like this sort of non-sense ;) ).  This
+// basically means: return the type that inherits the other type,
+// because if another type inherits the lowermost type, then it will
+// also inherit the other..
+const ObjectImpType* lowermost( const ObjectImpType* a, const ObjectImpType* b )
+{
+  if ( a->inherits( b ) ) return a;
+  assert( b->inherits( a ) );
+  return b;
+}
+
+// this function is part of the visit procedure really.  It is
+// factored out, because it recurses for cache ObjectImp's.  What this
+// does is, it makes sure that object o is calcable, by putting
+// appropriate Node's in mnodes..  po is o->parents() and pl contains
+// the location of objects that are already in mnodes and -1
+// otherwise..  -1 means we have to store their ObjectImp, unless
+// they're cache ObjectImp's etc.
+int ObjectHierarchy::storeObject( const Object* o, const Objects& po, std::vector<int>& pl,
+                                  std::map<const Object*, int>& seenmap )
+{
+  for ( uint i = 0; i < po.size(); ++i )
+  {
+    if ( pl[i] == -1 )
+    {
+      // we can't store cache ObjectImp's..
+      if ( po[i]->imp()->isCache() )
+      {
+        Objects parentos = po[i]->parents();
+        std::vector<int> parentlocs( parentos.size(), -1 );
+        pl[i] = storeObject( po[i], parentos, parentlocs, seenmap );
+      }
+      else
+      {
+        mnodes.push_back( new PushStackNode( po[i]->imp()->copy() ) );
+        pl[i] = mnumberofargs + mnodes.size() - 1;
+      };
+    }
+    else if ( (uint) pl[i] < mnumberofargs )
+    {
+      Object* parent = o->parents()[i];
+      Objects opl = o->parents();
+
+      margrequirements[pl[i]] =
+        lowermost( margrequirements[pl[i]],
+                   o->impRequirement( parent, opl ) );
+    };
+  };
+  if ( o->inherits( Object::ID_RealObject ) )
+    mnodes.push_back( new ApplyTypeNode( static_cast<const RealObject*>( o )->type(), pl ) );
+  else if ( o->inherits( Object::ID_PropertyObject ) )
+  {
+    assert( pl.size() == 1 );
+    int parent = pl.front();
+    Object* op = po.front();
+    assert( op );
+    uint propid = static_cast<const PropertyObject*>( o )->propId();
+    assert( propid < op->propertiesInternalNames().size() );
+    mnodes.push_back( new FetchPropertyNode( parent, op->propertiesInternalNames()[propid], propid ) );
+  }
+  else
+    assert( false );
+  seenmap[o] = mnumberofargs + mnodes.size() - 1;
+  return mnumberofargs + mnodes.size() - 1;
+}
