@@ -24,6 +24,7 @@
 #include "../kig/kig_part.h"
 #include "../kig/kig_view.h"
 #include "../misc/object_constructor.h"
+#include "../misc/coordinate_system.h"
 #include "../misc/kigpainter.h"
 #include "../misc/calcpaths.h"
 
@@ -34,26 +35,19 @@ static void redefinePoint( Object* mpt, const Coordinate& c, KigDocument& doc, c
 {
   ObjectFactory::instance()->redefinePoint( mpt, c, doc, w );
   mpt->calc( doc );
-};
+}
 
 ConstructMode::ConstructMode( KigDocument& d, const ObjectConstructor* ctor )
   : BaseMode( d ), mctor( ctor ),
     mpt( 0 )
 {
-  Objects pos = ObjectFactory::instance()->fixedPoint( Coordinate( 0, 0 ) );
-  d._addObjects( Objects( pos.begin(), pos.end() - 1 ) );
-  mpt = pos.back();
+  mpt = ObjectFactory::instance()->fixedPoint( Coordinate( 0, 0 ) );
   mpt->calc( d );
 }
 
 ConstructMode::~ConstructMode()
 {
-  // delete mpt and its obsolete parents..
-  Objects tmp( mpt );
-  Objects dp = deadParents( tmp );
-  mdoc._delObjects( dp );
   delete mpt;
-  delete_all( dp.begin(), dp.end() );
 }
 
 void ConstructMode::leftClickedObject(
@@ -69,10 +63,8 @@ void ConstructMode::leftClickedObject(
     mdoc.addObject( mpt );
     selectObject( mpt, w );
     // get a new mpt for our further use..
-    Objects pos = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ),
-                                                            mdoc, w );
-    mdoc._addObjects( Objects( pos.begin(), pos.end() - 1 ) );
-    mpt = pos.back();
+    mpt = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ),
+                                                             mdoc, w );
     mpt->calc( mdoc );
   }
   else
@@ -88,9 +80,7 @@ void ConstructMode::midClicked( const QPoint& p, KigWidget& w )
 
     selectObject( mpt, w );
 
-    Objects pos = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ), mdoc, w );
-    mdoc._addObjects( Objects( pos.begin(), pos.end() - 1 ) );
-    mpt = pos.back();
+    mpt = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ), mdoc, w );
     mpt->calc( mdoc );
   }
 }
@@ -102,7 +92,8 @@ void ConstructMode::rightClicked( const Objects&, const QPoint&, KigWidget& )
 
 void ConstructMode::mouseMoved( const Objects& os,
                                 const QPoint& p,
-                                KigWidget& w )
+                                KigWidget& w,
+                                bool shiftpressed )
 {
   w.updateCurPix();
   KigPainter pter( w.screenInfo(), &w.curPix, mdoc );
@@ -111,7 +102,11 @@ void ConstructMode::mouseMoved( const Objects& os,
   QPoint textloc = p;
   textloc.setX( textloc.x() + 15 );
 
-  redefinePoint( mpt, w.fromScreen( p ), mdoc, w );
+  Coordinate ncoord = w.fromScreen( p );
+  if ( shiftpressed )
+    ncoord = mdoc.coordinateSystem().snapToGrid( ncoord, w );
+
+  redefinePoint( mpt, ncoord, mdoc, w );
 
   if ( !os.empty() && !mparents.contains( os.front() ) &&
        mctor->wantArgs( mparents.with( os.front() ), mdoc, w ) )
@@ -170,23 +165,14 @@ PointConstructMode::PointConstructMode( KigDocument& d )
   // ObjectFactory::redefinePoint does that too, and this way, we can
   // depend on them already being known by the doc when we add the
   // mpt..
-  Objects pos = ObjectFactory::instance()->fixedPoint( Coordinate() );
-  d._addObjects( Objects( pos.begin(), pos.end() - 1 ) );
-  mpt = pos.back();
+  mpt = ObjectFactory::instance()->fixedPoint( Coordinate() );
   mpt->calc( d );
 }
 
 PointConstructMode::~PointConstructMode()
 {
   // delete mpt and its obsolete parents..
-  if ( mpt )
-  {
-    Objects tmp( mpt );
-    Objects dp = deadParents( tmp );
-    mdoc._delObjects( dp );
-    delete mpt;
-    delete_all( dp.begin(), dp.end() );
-  };
+  delete mpt;
 }
 
 void PointConstructMode::leftClickedObject(
@@ -212,12 +198,17 @@ void PointConstructMode::rightClicked( const Objects&, const QPoint&,
 void PointConstructMode::mouseMoved(
   const Objects&,
   const QPoint& p,
-  KigWidget& w )
+  KigWidget& w,
+  bool shiftpressed )
 {
   w.updateCurPix();
   KigPainter pter( w.screenInfo(), &w.curPix, mdoc );
 
-  redefinePoint( mpt, w.fromScreen( p ), mdoc, w );
+  Coordinate ncoord = w.fromScreen( p );
+  if ( shiftpressed )
+    ncoord = mdoc.coordinateSystem().snapToGrid( ncoord, w );
+
+  redefinePoint( mpt, ncoord, mdoc, w );
 
   mpt->draw( pter, true );
   w.setCursor( KCursor::blankCursor() );
