@@ -76,14 +76,15 @@ void DefineMacroMode::enableActions()
 
 void DefineMacroMode::givenPageEntered()
 {
-  for_each( mdoc.objects().begin(), mdoc.objects().end(),
+  Objects objs = mdoc.objects();
+  for_each( objs.begin(), objs.end(),
             bind2nd( mem_fun( &Object::setSelected ), false ) );
   for_each( mgiven.begin(), mgiven.end(),
             bind2nd( mem_fun( &Object::setSelected ), true ) );
   static_cast<KigView*>( mdoc.widget() )->realWidget()->redrawScreen();
 
   updateNexts();
-};
+}
 
 void DefineMacroMode::finalPageEntered()
 {
@@ -114,7 +115,8 @@ void DefineMacroMode::namePageEntered()
   using std::for_each;
   using std::bind2nd;
   using std::mem_fun;
-  for_each( mdoc.objects().begin(), mdoc.objects().end(),
+  Objects objs = mdoc.objects();
+  for_each( objs.begin(), objs.end(),
             bind2nd( mem_fun( &Object::setSelected ), false ) );
   static_cast<KigView*>( mdoc.widget() )->realWidget()->redrawScreen();
 
@@ -155,19 +157,21 @@ void DefineMacroMode::dragRect( const QPoint& p, KigWidget& w )
   Objects cos;
   DragRectMode dm( p, mdoc, w );
   mdoc.runMode( &dm );
-  Objects ret = dm.ret();
-  if ( dm.needClear() )
-  {
-    cos = *objs;
-    objs->setSelected( false );
-    objs->clear();
-  };
-  cos |= ret;
-  ret.setSelected( true );
-  objs->upush( ret );
-
   KigPainter pter( w.screenInfo(), &w.stillPix, mdoc );
-  pter.drawObjects( cos );
+  if ( ! dm.cancelled() )
+  {
+    Objects ret = dm.ret();
+    if ( dm.needClear() )
+    {
+      cos = *objs;
+      objs->setSelected( false );
+      objs->clear();
+    };
+    cos |= ret;
+    ret.setSelected( true );
+    objs->upush( ret );
+    pter.drawObjects( cos );
+  };
   w.updateCurPix( pter.overlay() );
   w.updateWidget();
 
@@ -198,7 +202,7 @@ void DefineMacroMode::leftClickedObject( Object* o, const QPoint&,
   updateNexts();
 }
 
-void DefineMacroMode::mouseMoved( const Objects& os, const QPoint& pt, KigWidget& w )
+void DefineMacroMode::mouseMoved( const Objects& os, const QPoint& pt, KigWidget& w, bool )
 {
   w.updateCurPix();
 
@@ -214,7 +218,8 @@ void DefineMacroMode::mouseMoved( const Objects& os, const QPoint& pt, KigWidget
     // and set statusbar text
 
     w.setCursor( KCursor::handCursor() );
-    QString selectstat = ObjectImp::selectStatement( os.front()->imp()->id() );
+    QString selectstat =
+      os.front()->imp()->type()->selectStatement();
 
     // statusbar text
     mdoc.emitStatusBarText( selectstat );
