@@ -32,17 +32,16 @@
 
 #include <functional>
 #include <algorithm>
+#include <map>
 
 void MovingModeBase::initScreen( const Objects& tin )
 {
-  // temporary fix before i finally fix the moving objects selection
-  // algorithm: don't try to move DataObjects..
+  // don't try to move objects that have been deleted from the
+  // document or internal objects that the user is not aware of..
   Objects in;
-  Objects docobjs = mdoc.objects();
+  Objects docobjs = mdoc.allObjects();
   for ( uint i = 0; i < tin.size(); ++i )
     if ( docobjs.contains( tin[i] ) )
-         // don't try to move objects that have been
-         // deleted from the document..
       in.push_back( tin[i] );
 
   // here we calc what objects will be moving, and we draw the others
@@ -53,7 +52,7 @@ void MovingModeBase::initScreen( const Objects& tin )
   amo = in;
   // calc nmo: basically ( mdoc.objects() - amo )
   // we use some stl magic here, tmp and tmp2 are set to os and
-  // mdoc->objects(), sorted, and then used in set_difference...
+  // mdoc.objects(), sorted, and then used in set_difference...
   Objects tmp( amo.begin(), amo.end() );
   sort( tmp.begin(), tmp.end() );
   Objects tmp2( mdoc.objects() );
@@ -90,29 +89,56 @@ void MovingModeBase::mouseMoved( QMouseEvent* e, KigWidget* v )
 {
   v->updateCurPix();
   Coordinate c = v->fromScreen( e->pos() );
-  moveTo( c );
+
+  bool snaptogrid = e->state() & Qt::ShiftButton;
+  moveTo( c, snaptogrid );
   amo.calc( mdoc );
   KigPainter p( v->screenInfo(), &v->curPix, mdoc );
   p.drawObjects( amo );
   v->updateWidget( p.overlay() );
   v->updateScrollBars();
+}
+
+class MovingMode::Private
+{
+public:
+  // explicitly moving objects: these are the objects that the user
+  // requested to move...
+  Objects emo;
+  // point where we started moving..
+  Coordinate pwwsm;
+  MonitorDataObjects* mon;
+  // we keep a map from the emo objects to their reference location.
+  // This is the location that they claim to be at before moving
+  // starts, and we use it as a reference point to determine where
+  // they should move next..
+  std::map<const Object*, Coordinate> refmap;
 };
 
 MovingMode::MovingMode( const Objects& os, const Coordinate& c,
                         KigWidget& v, KigDocument& doc )
-  : MovingModeBase( doc, v ), pwwlmt( c ),
-    mon( 0 )
+  : MovingModeBase( doc, v ), d( new Private )
 {
+  d->pwwsm = c;
   Objects objs;
+  Objects emo;
   for ( Objects::const_iterator i = os.begin(); i != os.end(); ++i )
     if ( (*i)->canMove() )
     {
       emo.upush( *i );
+      d->refmap[*i] = (*i)->moveReferencePoint();
       objs.upush( *i );
       objs |= getAllParents( Objects( *i ) );
     };
 
-  mon = new MonitorDataObjects( objs );
+  emo = calcPath( emo );
+  for ( Objects::const_iterator i = emo.begin(); i != emo.end(); ++i )
+    if ( !isChild( *i, d->emo ) )
+      d->emo.push_back( *i );
+
+  kdDebug() << k_funcinfo << d->emo.size() << endl;
+
+  d->mon = new MonitorDataObjects( objs );
 
   Objects tmp = objs;
   for ( Objects::const_iterator i = tmp.begin(); i != tmp.end(); ++i )
@@ -123,19 +149,23 @@ MovingMode::MovingMode( const Objects& os, const Coordinate& c,
 
 void MovingMode::stopMove()
 {
-  QString text = emo.size() == 1 ?
-                 ObjectImp::moveAStatement( emo[0]->imp()->id() ) :
-                 i18n( "Move %1 Objects" ).arg( emo.size() );
+  QString text = d->emo.size() == 1 ?
+                 d->emo[0]->imp()->type()->moveAStatement() :
+                 i18n( "Move %1 Objects" ).arg( d->emo.size() );
   KigCommand* mc = new KigCommand( mdoc, text );
-  mc->addTask( mon->finish() );
+  mc->addTask( d->mon->finish() );
   mdoc.history()->addCommand( mc );
 }
 
-void MovingMode::moveTo( const Coordinate& o )
+void MovingMode::moveTo( const Coordinate& o, bool snaptogrid )
 {
-  for( Objects::iterator i = emo.begin(); i != emo.end(); ++i )
-    (*i)->move( pwwlmt, o - pwwlmt, mdoc );
-  pwwlmt = o;
+  for( Objects::iterator i = d->emo.begin(); i != d->emo.end(); ++i )
+  {
+    assert( d->refmap.find( *i ) != d->refmap.end() );
+    Coordinate nc = d->refmap[*i] + ( o - d->pwwsm );
+    if ( snaptogrid ) nc = mdoc.coordinateSystem().snapToGrid( nc, mview );
+    (*i)->move( nc, mdoc );
+  };
 }
 
 PointRedefineMode::PointRedefineMode( RealObject* p, KigDocument& d, KigWidget& v )
@@ -150,9 +180,11 @@ PointRedefineMode::PointRedefineMode( RealObject* p, KigDocument& d, KigWidget&
   initScreen( os );
 }
 
-void PointRedefineMode::moveTo( const Coordinate& o )
+void PointRedefineMode::moveTo( const Coordinate& o, bool snaptogrid )
 {
-  (void) ObjectFactory::instance()->redefinePoint( mp, o, mdoc, mview );
+  Coordinate realo =
+    snaptogrid ? mdoc.coordinateSystem().snapToGrid( o, mview ) : o;
+  (void) ObjectFactory::instance()->redefinePoint( mp, realo, mdoc, mview );
 }
 
 PointRedefineMode::~PointRedefineMode()
@@ -171,17 +203,18 @@ MovingModeBase::~MovingModeBase()
 void MovingModeBase::leftMouseMoved( QMouseEvent* e, KigWidget* v )
 {
   mouseMoved( e, v );
-};
+}
 
 MovingMode::~MovingMode()
 {
-  delete mon;
+  delete d->mon;
+  delete d;
 }
 
 void PointRedefineMode::stopMove()
 {
-  Objects parents = mp->parents();
-  const ObjectType* type = mp->type();
+  ReferenceObject newparentsref( mp->parents() );
+  const ObjectType* newtype = mp->type();
   mp->setType( moldtype );
   mp->setParents( moldparents );
   mp->calc( mdoc );
@@ -192,7 +225,7 @@ void PointRedefineMode::stopMove()
 
   KigCommand* command = new KigCommand( mdoc, i18n( "Redefine Point" ) );
   command->addTask(
-    new ChangeParentsAndTypeTask( mp, parents, type ) );
+    new ChangeParentsAndTypeTask( mp, newparentsref.parents(), newtype ) );
   command->addTask( mmon->finish() );
   mdoc.history()->addCommand( command );
 }
