@@ -43,6 +43,9 @@ void NormalMode::enableActions()
 {
   KigMode::enableActions();
   mdoc.enableConstructActions( true );
+  mdoc.aSelectAll->setEnabled( true );
+  mdoc.aDeselectAll->setEnabled( true );
+  mdoc.aInvertSelection->setEnabled( true );
   mdoc.aDeleteObjects->setEnabled( true );
   mdoc.aShowHidden->setEnabled( true );
   mdoc.aNewMacro->setEnabled( true );
@@ -58,12 +61,11 @@ void NormalMode::deleteObjects()
 
 void NormalMode::selectObject( Object* o )
 {
-  assert( o->inherits( Object::ID_RealObject ) );
   sos.push_back( o );
-  static_cast<RealObject*>( o )->setSelected( true );
+  o->setSelected( true );
 }
 
-void NormalMode::selectObjects( Objects& os )
+void NormalMode::selectObjects( const Objects& os )
 {
   // hehe, don't you love this c++ stuff ;)
   for_each( os.begin(), os.end(),
@@ -91,11 +93,10 @@ void NormalMode::clearSelection()
 
 void NormalMode::showHidden()
 {
-  const Objects& os = mdoc.objects();
+  const Objects os = mdoc.objects();
   for (Objects::const_iterator i = os.begin(); i != os.end(); ++i )
-    if( !(*i)->isInternal() )
-      (*i)->setShown( true );
-  objectsAdded();
+    (*i)->setShown( true );
+  redrawScreen();
 }
 
 void NormalMode::newMacro()
@@ -104,16 +105,21 @@ void NormalMode::newMacro()
   mdoc.runMode( &m );
 }
 
-void NormalMode::objectsAdded()
+void NormalMode::redrawScreen()
 {
-  KigWidget* w = mdoc.mainWidget()->realWidget();
-  w->redrawScreen();
-  w->updateScrollBars();
-}
-
-void NormalMode::objectsRemoved()
-{
-  objectsAdded();
+  // unselect removed objects..
+  Objects nsos;
+  for ( uint i = 0; i < sos.size(); ++i )
+    if ( mdoc.objects().contains( sos[i] ) )
+      nsos.push_back( sos[i] );
+  sos = nsos;
+  const std::vector<KigWidget*>& widgets = mdoc.widgets();
+  for ( uint i = 0; i < widgets.size(); ++i )
+  {
+    KigWidget* w = widgets[i];
+    w->redrawScreen();
+    w->updateScrollBars();
+  };
 }
 
 void NormalMode::editTypes()
@@ -136,19 +142,23 @@ void NormalMode::dragRect( const QPoint& p, KigWidget& w )
   DragRectMode d( p, mdoc, w );
   mdoc.runMode( &d );
 
-  Objects sel = d.ret();
-  Objects cos = sel;
+  KigPainter pter( w.screenInfo(), &w.stillPix, mdoc );
 
-  if ( d.needClear() )
+  if ( ! d.cancelled() )
   {
-    cos |= sos;
-    clearSelection();
+    Objects sel = d.ret();
+    Objects cos = sel;
+
+    if ( d.needClear() )
+    {
+      cos |= sos;
+      clearSelection();
+    };
+
+    selectObjects( sel );
+    pter.drawObjects( cos );
   };
 
-  selectObjects( sel );
-
-  KigPainter pter( w.screenInfo(), &w.stillPix, mdoc );
-  pter.drawObjects( cos );
   w.updateCurPix( pter.overlay() );
   w.updateWidget();
 }
@@ -157,7 +167,7 @@ void NormalMode::dragObject( const Objects& oco, const QPoint& pco,
                              KigWidget& w, bool ctrlOrShiftDown )
 {
   // first determine what to move...
-  if( ( oco & sos ).empty() )
+  if( !sos.contains( oco.front() ) )
   {
     // the user clicked on something that is currently not
     // selected... --> we select it, taking the Ctrl- and
@@ -205,20 +215,20 @@ void NormalMode::leftClickedObject( Object* o, const QPoint&,
 
 void NormalMode::midClicked( const QPoint& p, KigWidget& w )
 {
-  Objects ptos = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ), mdoc, w );
-  ptos.calc( mdoc );
-  mdoc.addObjects( ptos );
+  Object* pto = ObjectFactory::instance()->sensiblePoint( w.fromScreen( p ), mdoc, w );
+  pto->calc( mdoc );
+  mdoc.addObject( pto );
 
   // refresh the screen...
-  w.redrawScreen();
-  w.updateScrollBars();
+  // not necessary, done by addObjects, which calls NormalMode::redrawScreen..
+//  w.redrawScreen();
+//   w.updateScrollBars();
 }
 
 void NormalMode::rightClicked( const Objects& os,
                                const QPoint&,
                                KigWidget& w )
 {
-  // TODO
   if( !os.empty() )
   {
     if( !sos.contains( os.front() ) )
@@ -233,15 +243,15 @@ void NormalMode::rightClicked( const Objects& os,
   }
   else
   {
-//     KigDocumentPopup* m = popup( mdoc );
-//     if( m ) m->exec( w.mapToGlobal( plc ) );
-//     delete m;
+    NormalModePopupObjects p( mdoc, w, *this, Objects() );
+    p.exec( QCursor::pos() );
   };
 }
 
 void NormalMode::mouseMoved( const Objects& os,
                              const QPoint& plc,
-                             KigWidget& w )
+                             KigWidget& w,
+                             bool )
 {
   w.updateCurPix();
   if( os.empty() )
@@ -257,7 +267,7 @@ void NormalMode::mouseMoved( const Objects& os,
 
     w.setCursor( KCursor::handCursor() );
 
-    QString stat = i18n( ObjectImp::selectStatement( os.front()->imp()->id() ) );
+    QString stat = i18n( os.front()->imp()->type()->selectStatement() );
 
     // statusbar text
     mdoc.emitStatusBarText( stat );
@@ -271,3 +281,31 @@ void NormalMode::mouseMoved( const Objects& os,
     w.updateWidget( p.overlay() );
   };
 }
+
+void NormalMode::selectAll()
+{
+  const Objects os = mdoc.objects();
+  selectObjects( os );
+  redrawScreen();
+}
+
+void NormalMode::deselectAll()
+{
+  clearSelection();
+  redrawScreen();
+}
+
+void NormalMode::invertSelection()
+{
+  Objects sel = sos;
+  Objects os = mdoc.objects();
+  clearSelection();
+  for ( Objects::const_iterator i = os.begin();
+        i != os.end(); ++i )
+    if ( ! sel.contains( *i ) )
+    {
+      (*i)->setSelected( true );
+      sos.push_back( *i );
+    }
+  redrawScreen();
+}
