@@ -26,6 +26,8 @@
 #include "../kig/kig_commands.h"
 #include "../misc/i18n.h"
 #include "../objects/object_imp.h"
+#include "../objects/bogus_imp.h"
+#include "../objects/point_imp.h"
 #include "../objects/object.h"
 #include "../objects/other_type.h"
 #include "../objects/object_factory.h"
@@ -145,7 +147,7 @@ NormalModePopupObjects::NormalModePopupObjects( KigDocument& doc,
   connect( this, SIGNAL( activated( int ) ), this, SLOT( toplevelMenuSlot( int ) ) );
 
   insertTitle( empty ? i18n( "Kig Document" )
-               : single ? ObjectImp::translatedName( objs[0]->imp()->id() )
+               : single ? objs[0]->imp()->type()->translatedName()
                : i18n( "%1 Objects" ).arg( objs.size() ), 1 );
 
   if ( empty )
@@ -197,7 +199,7 @@ NormalModePopupObjects::NormalModePopupObjects( KigDocument& doc,
       i18n( "&Transform" ),
       i18n( "Const&ruct" ),
       i18n( "&Start" ),
-      i18n( "Sho&w" ),
+      i18n( "Add Te&xt Label" ),
       i18n( "Set Co&lor" ),
       i18n( "Set Si&ze" ),
       QString::null,
@@ -292,7 +294,7 @@ void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, in
     QColor color = os.front()->color();
     for ( Objects::const_iterator i = os.begin(); i != os.end(); ++i )
     {
-      if ( ! (*i)->imp()->inherits( ObjectImp::ID_PointImp ) )
+      if ( ! (*i)->imp()->inherits( PointImp::stype() ) )
         point = false;
       if ( (*i)->color() != color ) samecolor = false;
     };
@@ -466,6 +468,8 @@ void NormalModePopupObjects::addAction( int menu, const QPixmap& pix, int id )
   else m = mmenus[menu];
   int ret = m->insertItem( pix, id );
   assert( ret == id );
+  // pretend to use this var..
+  (void) ret;
 }
 
 void NormalModePopupObjects::setColorMenuSlot( int i )
@@ -490,6 +494,8 @@ void NormalModePopupObjects::addAction( int menu, const QPixmap& icon, const QSt
   else m = mmenus[menu];
   int ret = m->insertItem( QIconSet( icon ), name, id );
   assert( ret == id );
+  // pretend to use this var..
+  (void)ret;
 }
 
 void NormalModePopupObjects::addAction( int menu, const QString& name, int id )
@@ -499,6 +505,8 @@ void NormalModePopupObjects::addAction( int menu, const QString& name, int id )
   else m = mmenus[menu];
   int ret = m->insertItem( name, id );
   assert( ret == id );
+  // pretend to use this var..
+  (void)ret;
 }
 
 PopupActionProvider::~PopupActionProvider()
@@ -522,12 +530,12 @@ void PropertiesActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
     {
       // we don't want imp's like DoubleImp, since we can't show them
       // anyway..
-      add &= ! prop->inherits( ObjectImp::ID_BogusImp );
+      add &= ! prop->inherits( BogusImp::stype() );
       // we don't want to construct PointImp's coordinate property,
       // since it would construct a point at the same place as its
       // parent..
-      add &= ! ( o->hasimp( ObjectImp::ID_PointImp ) &&
-                 prop->inherits( ObjectImp::ID_PointImp ) );
+      add &= ! ( o->hasimp( PointImp::stype() ) &&
+                 prop->inherits( PointImp::stype() ) );
     }
     else if ( menu == NormalModePopupObjects::ShowMenu )
       add &= prop->canFillInNextEscape();
@@ -568,13 +576,12 @@ bool PropertiesActionsProvider::executeAction(
   {
     Objects ret;
     ret.push_back( new PropertyObject( parent, propid ) );
+    ret.back()->calc( doc );
     Coordinate c = w.fromScreen( w.mapFromGlobal( popup.mapToGlobal( QPoint( 5, 0 ) ) ) );
-    Objects labelos = ObjectFactory::instance()->label(
-      QString::fromLatin1( "%1" ), c,
-      false, ret );
-    copy( labelos.begin(), labelos.end(), back_inserter( ret ) );
-    ret.calc( doc );
-    doc.addObjects( ret );
+    Object* label = ObjectFactory::instance()->attachedLabel(
+      QString::fromLatin1( "%1" ), parent, c,
+      false, ret, doc );
+    doc.addObject( label );
   }
   else
   {
@@ -630,21 +637,24 @@ void BuiltinDocumentActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
     popup.addAction( menu, p, i18n( "Zoom &In" ), nextfree++ );
     p = l->loadIcon( "viewmag-", KIcon::Toolbar );
     popup.addAction( menu, p, i18n( "Zoom &Out" ), nextfree++ );
-    if ( popup.widget().isFullScreen() )
-      popup.addAction( menu, i18n( "E&xit Full Screen Mode" ), nextfree++ );
+    p = l->loadIcon( "window_fullscreen", KIcon::User );
+    popup.addAction( menu, p, i18n( "Toggle &Full Screen Mode" ), nextfree++ );
   }
   else if ( menu == NormalModePopupObjects::SetCoordinateSystemMenu )
   {
+    int idoffset = nextfree;
     QStringList l = CoordinateSystemFactory::names();
     mnumberofcoordsystems = l.count();
     for ( uint i = 0; i < l.count(); ++i )
       popup.addAction( menu, l[i], nextfree++ );
+    int current = popup.document().coordinateSystem().id();
+    popup.setChecked( menu, idoffset + current, true );
   }
 }
 
 bool BuiltinDocumentActionsProvider::executeAction(
   int menu, int& id, const Objects&,
-  NormalModePopupObjects& popup,
+  NormalModePopupObjects&,
   KigDocument& doc, KigWidget& w, NormalMode& m )
 {
   if ( menu == NormalModePopupObjects::ToplevelMenu )
@@ -658,26 +668,20 @@ bool BuiltinDocumentActionsProvider::executeAction(
     }
     else if ( id == 1 )
     {
-      w.zoomIn();
+      w.slotZoomIn();
       return true;
     }
     else if ( id == 2 )
     {
-      w.zoomOut();
+      w.slotZoomOut();
       return true;
     }
-    else if ( popup.widget().isFullScreen() )
+    else if ( id == 3 )
     {
-      if ( id == 3 )
-      {
-        assert( w.view()->parent()->inherits( "QDialog" ) );
-        static_cast<QDialog*>( w.view()->parent() )->close();
-        return true;
-      }
-      else
-        id -= 1;
-    };
-    id -= 3;
+      w.view()->toggleFullScreen();
+      return true;
+    }
+    id -= 4;
     return false;
   }
   else if ( menu == NormalModePopupObjects::SetCoordinateSystemMenu )
@@ -696,3 +700,8 @@ bool BuiltinDocumentActionsProvider::executeAction(
   else return false;
 }
 
+void NormalModePopupObjects::setChecked( int menu, int n, bool checked )
+{
+  mmenus[menu]->setItemChecked( n, checked );
+}
+
