@@ -26,46 +26,54 @@
 #include <qstring.h>
 
 /**
- * These ObjectImp's are not really in that they don't represent
- * objects.  They exist because ObjectImp's also serve another
- * purpose, namely containing data.  They can be loaded and saved, and
- * the Object class contains some fixedArgs, that are fixed for the
- * object at hand ( e.g. the Coordinates of a fixed point ).  These
- * ObjectImp's are pure data, and serve only to be loaded and saved..
+ * This is the base class for the so-called BogusImp's.  These
+ * ObjectImp's are not really ObjectImp's, in that they don't
+ * represent objects.  They exist because ObjectImp's also serve
+ * another purpose, namely containing data.  They can all be loaded
+ * and saved, and the only difference between these objects and normal
+ * objects are that these serve *only* to be loaded and saved.  This
+ * approach adds a lot of flexibility to the Kig system, and has
+ * certainly proven itself very valuable.
  */
 class BogusImp
   : public ObjectImp
 {
   typedef ObjectImp Parent;
 public:
+  /**
+   * Returns the ObjectImpType representing the BogusImp type.
+   */
+  static const ObjectImpType* stype();
+
   void draw( KigPainter& p ) const;
   bool contains( const Coordinate& p, int width, const KigWidget& w ) const;
   bool inRect( const Rect& r, int width, const KigWidget& w ) const;
-  bool valid() const;
 
   ObjectImp* transform( const Transformation& ) const;
-
-  bool inherits( int type ) const;
-  const uint numberOfProperties() const;
-  const QCStringList properties() const;
-  const QCStringList propertiesInternalNames() const;
-  ObjectImp* property( uint which, const KigDocument& d ) const;
-  const char* iconForProperty( uint which ) const;
-  int impRequirementForProperty( uint which ) const;
 };
 
+/**
+ * This ObjectImp represents an invalid object.  If a calculation
+ * fails, then often an InvalidImp is returned, indicating that the
+ * generated object is invalid.
+ */
 class InvalidImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
 public:
-  InvalidImp();
-  bool valid() const;
-  bool inherits( int t ) const;
-  InvalidImp* copy() const;
-  const char* baseName() const;
+  /**
+   * Returns the ObjectImpType representing the InvalidImp type.
+   */
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
 
-  int id() const;
+  /**
+   * Construct a new InvalidImp.
+   */
+  InvalidImp();
+  InvalidImp* copy() const;
+
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool canFillInNextEscape() const;
@@ -74,23 +82,37 @@ public:
   bool equals( const ObjectImp& rhs ) const;
 };
 
+/**
+ * This ObjectImp is a BogusImp containing only a double value.
+ */
 class DoubleImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
   double mdata;
 public:
+  /**
+   * Returns the ObjectImpType representing the DoubleImp type.
+   */
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
+
+  /**
+   * Construct a new DoubleImp containing the value d.
+   */
   DoubleImp( const double d );
 
+  /**
+   * Get hold of the contained data.
+   */
   double data() const { return mdata; };
+  /**
+   * Set the contained data to d.
+   */
   void setData( double d ) { mdata = d; };
 
-  bool inherits( int typeID ) const;
-
   DoubleImp* copy() const;
-  const char* baseName() const;
 
-  int id() const;
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool canFillInNextEscape() const;
@@ -99,22 +121,37 @@ public:
   bool equals( const ObjectImp& rhs ) const;
 };
 
+/**
+ * This ObjectImp is a BogusImp containing only an int value.
+ */
 class IntImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
   int mdata;
 public:
+  /**
+   * Returns the ObjectImpType representing the IntImp type..
+   */
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
+
+  /**
+   * Construct a new IntImp containing the value d.
+   */
   IntImp( const int d );
 
+  /**
+   * Get hold of the contained data.
+   */
   int data() const { return mdata; };
+  /**
+   * Set the contained data to d.
+   */
   void setData( int d )  { mdata = d; }
 
-  bool inherits( int typeID ) const;
   IntImp* copy() const;
-  const char* baseName() const;
 
-  int id() const;
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool canFillInNextEscape() const;
@@ -123,22 +160,37 @@ public:
   bool equals( const ObjectImp& rhs ) const;
 };
 
+/**
+ * This ObjectImp is a BogusImp containing only a string value.
+ */
 class StringImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
   QString mdata;
 public:
+  /**
+   * Returns the ObjectImpType representing the StringImp type..
+   */
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
+
+  /**
+   * Construct a new StringImp containing the string d.
+   */
   StringImp( const QString& d );
 
+  /**
+   * Get hold of the contained data.
+   */
   const QString& data() const { return mdata; };
+  /**
+   * Set the contained data.
+   */
   void setData( const QString& s ) { mdata = s; }
 
-  bool inherits( int typeID ) const;
   StringImp* copy() const;
-  const char* baseName() const;
 
-  int id() const;
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool canFillInNextEscape() const;
@@ -150,47 +202,48 @@ public:
 class HierarchyImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
   ObjectHierarchy mdata;
 public:
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
+
   HierarchyImp( const ObjectHierarchy& h );
 
   const ObjectHierarchy& data() const { return mdata; };
   void setData( const ObjectHierarchy& h ) { mdata = h; };
 
-  bool inherits( int type ) const;
   HierarchyImp* copy() const;
   const char* baseName() const;
 
-  int id() const;
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool equals( const ObjectImp& rhs ) const;
 };
 
 /**
- * Don't mistake this imp for something that draws a transformed
- * object.  It does something completely different.  It's a pure data
- * Imp, like DoubleImp and friends that serves only to store the data
- * of a transformation ( see the Transformation class in
- * ../misc/kigtransform.h
+ * \internal Don't mistake this imp for something that draws a
+ * transformed object.  It does something completely different.  It's
+ * a pure data Imp, like DoubleImp and friends that serves only to
+ * store the data of a transformation ( see the Transformation class
+ * in ../misc/kigtransform.h
  */
 class TransformationImp
   : public BogusImp
 {
-  typedef BogusImp Parent;
   Transformation mdata;
 public:
+  static const ObjectImpType* stype();
+  typedef BogusImp Parent;
+
   TransformationImp( const Transformation& h );
 
   const Transformation& data() const { return mdata; };
   void setData( const Transformation& h ) { mdata = h; };
 
-  bool inherits( int type ) const;
   TransformationImp* copy() const;
-  const char* baseName() const;
 
-  int id() const;
+  const ObjectImpType* type() const;
   void visit( ObjectImpVisitor* vtor ) const;
 
   bool equals( const ObjectImp& rhs ) const;
