@@ -24,6 +24,7 @@
 #include "../misc/object_hierarchy.h"
 #include "../misc/kigpainter.h"
 #include "../misc/coordinate.h"
+#include "../misc/common.h"
 
 #include "../kig/kig_view.h"
 
@@ -33,18 +34,11 @@
 
 using namespace std;
 
-static const double double_inf = HUGE_VAL;
-
 LocusImp::~LocusImp()
 {
   delete mcurve;
 }
 
-bool LocusImp::inherits( int type ) const
-{
-  return type == ID_LocusImp ? true : Parent::inherits( type );
-}
-
 ObjectImp* LocusImp::transform( const Transformation& t ) const
 {
   return new LocusImp( mcurve->copy(), mhier.transformFinalObject( t ) );
@@ -52,7 +46,7 @@ ObjectImp* LocusImp::transform( const Transformation& t ) const
 
 void LocusImp::draw( KigPainter& p ) const
 {
-  p.drawLocus( mcurve, mhier );
+  p.drawCurve( this );
 }
 
 bool LocusImp::contains( const Coordinate& p, int width, const KigWidget& w ) const
@@ -68,11 +62,6 @@ bool LocusImp::inRect( const Rect&, int, const KigWidget& ) const
   return false;
 }
 
-bool LocusImp::valid() const
-{
-  return true;
-}
-
 const Coordinate LocusImp::getPoint( double param, bool& valid, const KigDocument& doc ) const
 {
   Coordinate arg = mcurve->getPoint( param, valid, doc );
@@ -84,7 +73,7 @@ const Coordinate LocusImp::getPoint( double param, bool& valid, const KigDocumen
   assert( calcret.size() == 1 );
   ObjectImp* imp = calcret.front();
   Coordinate ret;
-  if ( imp->inherits( ObjectImp::ID_PointImp ) )
+  if ( imp->inherits( PointImp::stype() ) )
   {
     valid = true;
     ret = static_cast<PointImp*>( imp )->coordinate();
@@ -116,11 +105,16 @@ const QCStringList LocusImp::properties() const
   return Parent::properties();
 }
 
-int LocusImp::impRequirementForProperty( uint which ) const
+const ObjectImpType* LocusImp::impRequirementForProperty( uint which ) const
 {
   return Parent::impRequirementForProperty( which );
 }
 
+const char* LocusImp::iconForProperty( uint which ) const
+{
+  return Parent::iconForProperty( which );
+}
+
 ObjectImp* LocusImp::property( uint which, const KigDocument& w ) const
 {
   return Parent::property( which, w );
@@ -131,11 +125,6 @@ LocusImp* LocusImp::copy() const
   return new LocusImp( mcurve->copy(), mhier );
 }
 
-const char* LocusImp::baseName() const
-{
-  return I18N_NOOP( "locus" );
-}
-
 const CurveImp* LocusImp::curve() const
 {
   return mcurve;
@@ -146,11 +135,6 @@ const ObjectHierarchy& LocusImp::hierarchy() const
   return mhier;
 }
 
-int LocusImp::id() const
-{
-  return ID_LocusImp;
-}
-
 /**
  * This function returns the distance between the point with parameter
  * param and point p.  param is allowed to not be between 0 and 1, in
@@ -198,7 +182,7 @@ void LocusImp::getInterval( double& x1, double& x2,
 
 double LocusImp::getParam( const Coordinate& p, const KigDocument& doc ) const
 {
-  // thiis function ( and related functions like getInterval etc. ) is
+  // this function ( and related functions like getInterval etc. ) is
   // written by Franco Pasquarelli <pasqui@dmf.bs.unicatt.it>.
   // I ( domi ) have adapted and documented it a bit.
 
@@ -331,3 +315,29 @@ void LocusImp::visit( ObjectImpVisitor* vtor ) const
 {
   vtor->visit( this );
 }
+
+bool LocusImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( LocusImp::stype() ) &&
+    static_cast<const LocusImp&>( rhs ).curve()->equals( *curve() ) &&
+    static_cast<const LocusImp&>( rhs ).hierarchy() == hierarchy();
+}
+
+const ObjectImpType* LocusImp::stype()
+{
+  static const ObjectImpType t(
+    Parent::stype(), "locus",
+    I18N_NOOP( "locus" ),
+    I18N_NOOP( "Select this locus" ),
+    I18N_NOOP( "Remove a Locus" ),
+    I18N_NOOP( "Add a Locus" ),
+    I18N_NOOP( "Move a Locus" ),
+    I18N_NOOP( "Attach to this locus" )
+    );
+  return &t;
+}
+
+const ObjectImpType* LocusImp::type() const
+{
+  return LocusImp::stype();
+}
