@@ -65,10 +65,9 @@ bool RealObject::inRect( const Rect& r, const KigWidget& w ) const
   return mimp->inRect( r, mwidth, w );
 }
 
-void RealObject::move( const Coordinate& from, const Coordinate& dist,
-                       const KigDocument& d )
+void RealObject::move( const Coordinate& to, const KigDocument& d )
 {
-  mtype->move( this, from, dist, d );
+  mtype->move( this, to, d );
 }
 
 void ObjectWithParents::calc( const KigDocument& d )
@@ -88,13 +87,10 @@ bool RealObject::canMove() const
 
 Object::~Object()
 {
-  // tell our children that we're dead, so they don't try to tell us
-  // that they're dying too, which would cause segfaults...
-  for ( uint i = 0; i < mchildren.size(); ++i )
-    mchildren[i]->delParent( this );
+  assert( mchildren.empty() );
 }
 
-bool Object::hasimp( int type ) const
+bool Object::hasimp( const ObjectImpType* type ) const
 {
   assert( imp() );
   return imp()->inherits( type );
@@ -120,7 +116,7 @@ const QCStringList Object::properties() const
   return imp()->properties();
 }
 
-const Objects& Object::children() const
+const Objects Object::children() const
 {
   return mchildren;
 }
@@ -148,12 +144,22 @@ const Objects Object::getAllChildren() const
 
 void Object::addChild( Object* o )
 {
-  mchildren.upush( o );
+  mchildren.push_back( o );
 }
 
 void Object::delChild( Object* o )
 {
-  mchildren.remove( o );
+  // the amount of children we have works like a reference count.  If
+  // there are none, we commit suicide..
+  mchildren.remove_first( o );
+  if ( mchildren.empty() )
+    delete this;
+}
+
+void ObjectWithParents::addParents( const Objects& os )
+{
+  for ( Objects::const_iterator i = os.begin(); i != os.end(); ++i )
+    addParent( *i );
 }
 
 void ObjectWithParents::addParent( Object* o )
@@ -165,15 +171,23 @@ void ObjectWithParents::addParent( Object* o )
 void ObjectWithParents::delParent( Object* o )
 {
   mparents.remove( o );
+  o->delChild( this );
+}
+
+void ReferenceObject::clearParents()
+{
+  for ( Objects::iterator i = mparents.begin(); i != mparents.end(); ++i )
+    (*i)->delChild( this );
+  mparents.clear();
 }
 
 void ObjectWithParents::setParents( const Objects& parents )
 {
+  for ( uint i = 0; i < parents.size(); ++i )
+    parents[i]->addChild( this );
   for ( uint i = 0; i < mparents.size(); ++i )
     mparents[i]->delChild( this );
   mparents = parents;
-  for ( uint i = 0; i < mparents.size(); ++i )
-    mparents[i]->addChild( this );
 }
 
 Objects ObjectWithParents::parents() const
@@ -256,16 +270,6 @@ void DataObject::calc( const KigDocument& )
 {
 }
 
-void Object::addParent( Object* )
-{
-  assert( false );
-}
-
-void Object::delParent( Object* )
-{
-  assert( false );
-}
-
 void Object::setParents( const Objects& )
 {
   assert( false );
@@ -337,16 +341,6 @@ void RealObject::setShown( bool shown )
   mshown = shown;
 }
 
-bool RealObject::isInternal() const
-{
-  return false;
-}
-
-bool DataObject::isInternal() const
-{
-  return true;
-}
-
 PropertyObject::PropertyObject( Object* parent, int id )
   : Object(), mimp( new InvalidImp ), mparent( parent ), mpropid( id )
 {
@@ -355,7 +349,7 @@ PropertyObject::PropertyObject( Object* parent, int id )
 
 PropertyObject::~PropertyObject()
 {
-  if ( mparent ) mparent->delChild( this );
+  mparent->delChild( this );
   delete mimp;
 }
 
@@ -364,11 +358,6 @@ bool PropertyObject::inherits( int type ) const
   return type == ID_PropertyObject;
 }
 
-bool PropertyObject::isInternal() const
-{
-  return true;
-}
-
 const ObjectImp* PropertyObject::imp() const
 {
   return mimp;
@@ -415,14 +404,6 @@ int PropertyObject::propId() const
   return mpropid;
 }
 
-void PropertyObject::delParent( Object* o )
-{
-  // if a parent gets deleted before us, we set mparent to zero, and
-  // wait to be deleted ourselves ( which will be immediately after
-  // this )
-  if ( mparent == o ) mparent = 0;
-}
-
 void Object::setColor( const QColor& )
 {
 }
@@ -437,8 +418,7 @@ bool Object::canMove() const
   return false;
 }
 
-void Object::move( const Coordinate&, const Coordinate&,
-                   const KigDocument& )
+void Object::move( const Coordinate&, const KigDocument& )
 {
   assert( false );
 }
@@ -446,6 +426,7 @@ void Object::move( const Coordinate&, const Coordinate&,
 QColor Object::color() const
 {
   assert( false );
+  return Qt::black;
 }
 
 QColor RealObject::color() const
@@ -458,7 +439,7 @@ void RealObject::setWidth( int width )
   mwidth = width;
 }
 
-int RealObject::impRequirement( Object* o, const Objects& os ) const
+const ObjectImpType* RealObject::impRequirement( Object* o, const Objects& os ) const
 {
   Args args;
   args.reserve( mparents.size() );
@@ -468,17 +449,18 @@ int RealObject::impRequirement( Object* o, const Objects& os ) const
   return mtype->impRequirement( o->imp(), args );
 }
 
-int Object::impRequirement( Object*, const Objects& ) const
+const ObjectImpType* Object::impRequirement( Object*, const Objects& ) const
 {
   assert( false );
+  return ObjectImp::stype();
 }
 
-int PropertyObject::impRequirement( Object*, const Objects& ) const
+const ObjectImpType* PropertyObject::impRequirement( Object*, const Objects& ) const
 {
   return mparent->impRequirementForProperty( mpropid );
 }
 
-int Object::impRequirementForProperty( uint which ) const
+const ObjectImpType* Object::impRequirementForProperty( uint which ) const
 {
   return imp()->impRequirementForProperty( which );
 }
@@ -498,6 +480,11 @@ Object* PropertyObject::parent()
   return mparent;
 }
 
+ReferenceObject::ReferenceObject( Object* o )
+  : ObjectWithParents( Objects( o ) )
+{
+}
+
 ReferenceObject::ReferenceObject( const Objects& os )
   : ObjectWithParents( os )
 {
@@ -507,12 +494,6 @@ ReferenceObject::~ReferenceObject()
 {
 }
 
-bool ReferenceObject::isInternal() const
-{
-  // pretend to not be internal, so we don't get deleted ourselves..
-  return false;
-}
-
 const ObjectImp* ReferenceObject::imp() const
 {
   return new InvalidImp;
@@ -543,3 +524,17 @@ bool ReferenceObject::contains( const Coordinate&, const KigWidget& ) const
   return false;
 }
 
+ReferenceObject::ReferenceObject()
+  : ObjectWithParents( Objects() )
+{
+}
+
+const Coordinate Object::moveReferencePoint() const
+{
+  return Coordinate::invalidCoord();
+}
+
+const Coordinate RealObject::moveReferencePoint() const
+{
+  return mtype->moveReferencePoint( this );
+}
