@@ -22,6 +22,8 @@
 #include <qcolor.h>
 #include "common.h"
 
+class ObjectImpType;
+
 /**
  * The Object class is an ABC that represents a separate piece of the
  * Kig document.  This includes the user-perceived RealObjects, but
@@ -48,7 +50,6 @@ public:
   };
 
   virtual bool inherits( int type ) const;
-  virtual bool isInternal() const = 0;
 
   virtual const ObjectImp* imp() const = 0;
   virtual Objects parents() const = 0;
@@ -58,15 +59,21 @@ public:
    * @see ObjectType::impRequirement and
    * ObjectImp::impRequirementForProperty
    */
-  virtual int impRequirement( Object* o, const Objects& os ) const;
+  virtual const ObjectImpType* impRequirement( Object* o, const Objects& os ) const;
 
   virtual void draw( KigPainter& p, bool showSelection ) const = 0;
   virtual bool contains( const Coordinate& p, const KigWidget& si ) const = 0;
   virtual bool inRect( const Rect& r, const KigWidget& si ) const = 0;
 
   virtual bool canMove() const;
-  virtual void move( const Coordinate& from, const Coordinate& dist,
-                     const KigDocument& );
+  // what point should MovingMode think this object is currently on ?
+  // If the user then moves his mouse by (x,y), then move() will be
+  // called with to set to moveReferencePoint + (x,y)..
+  // a point should just return its current location, a segment should
+  // probably return the coordinate of one of its points, or the
+  // center or whatever..  just make sure your object moves properly ;)
+  virtual const Coordinate moveReferencePoint() const;
+  virtual void move( const Coordinate& to, const KigDocument& );
 
   virtual void calc( const KigDocument& ) = 0;
 
@@ -83,35 +90,32 @@ public:
   // for..
 
   // -> ObjectImp::inherits()
-  bool hasimp( int type ) const;
+  bool hasimp( const ObjectImpType* type ) const;
 
   bool valid() const;
 
   const uint numberOfProperties() const;
   ObjectImp* property( uint which, const KigDocument& w ) const;
-  int impRequirementForProperty( uint which ) const;
+  const ObjectImpType* impRequirementForProperty( uint which ) const;
+  const char* iconForProperty( uint which ) const;
   const QCStringList properties() const;
   const QCStringList propertiesInternalNames() const;
 
   // every kind of object can have children, and there is no easier
   // way of knowing which they are than to store their pointers.
   // Thus, it is the Object class's job to keep track of them..
-  const Objects& children() const;
+  const Objects children() const;
   const Objects getAllChildren() const;
 
   void addChild( Object* o );
 
   void delChild( Object* o );
 
-  virtual void addParent( Object* o );
-  virtual void delParent( Object* o );
-  // This function detects if our old parents contained internal
-  // objects that have no more children, removes them from the
-  // doc, and deletes them..  If you're (completely) sure that no old
-  // parents will have to be deleted, you can pass 0 as the document,
-  // e.g. if you know that no parents are know by the doc yet, or
-  // there are no parents at all yet..
-  virtual void setParents( const Objects& parents, KigDocument* doc );
+  // this function sets the parents of this object to parents.  It
+  // doesn't do anything for deleting obsolete parents etc. you should
+  // do this yourself ( hint: check out the deadParents() function in
+  // ../misc/calcpaths.h )
+  virtual void setParents( const Objects& parents );
 };
 
 class ObjectWithParents
@@ -124,8 +128,9 @@ protected:
   ~ObjectWithParents();
 public:
   void addParent( Object* o );
+  void addParents( const Objects& os );
   void delParent( Object* o );
-  void setParents( const Objects& parents, KigDocument* doc );
+  void setParents( const Objects& parents );
   Objects parents() const;
 
   void calc( const KigDocument& );
@@ -133,8 +138,8 @@ public:
 };
 
 /**
- * The Object class represents a geometric object.  This class
- * contains some information about the object, like its parents,
+ * The RealObject class represents an object that is shown on screen.
+ * This class contains some information about the object, like its
  * color, selected state etc., and pointers to an @ref ObjectType and
  * @ref ObjectImp.  Please check their respective documentation to see
  * what they're about..
@@ -164,7 +169,6 @@ public:
   void calc( const KigDocument& );
 
   bool inherits( int type ) const;
-  bool isInternal() const;
 
   const ObjectImp* imp() const;
 
@@ -173,7 +177,8 @@ public:
   bool inRect( const Rect& r, const KigWidget& ) const;
 
   bool canMove() const;
-  void move( const Coordinate& from, const Coordinate& dist, const KigDocument& );
+  const Coordinate moveReferencePoint() const;
+  void move( const Coordinate& dist, const KigDocument& );
 
   const ObjectType* type() const;
   void setType( const ObjectType* t );
@@ -190,7 +195,7 @@ public:
   int width() const { return mwidth; };
   void setWidth( int width );
 
-  int impRequirement( Object* o, const Objects& os ) const;
+  const ObjectImpType* impRequirement( Object* o, const Objects& os ) const;
 };
 
 /**
@@ -206,8 +211,12 @@ public:
   ~DataObject();
 
   bool inherits( int type ) const;
-  bool isInternal() const;
   void setImp( ObjectImp* imp );
+  /**
+   * sets imp as the old imp, and returns the old one.. This exchanges
+   * ownership in order to prevent some useless copies and deletions..
+   */
+  ObjectImp* switchImp( ObjectImp* imp );
 
   const ObjectImp* imp() const;
   Objects parents() const;
@@ -232,12 +241,12 @@ public:
   ~PropertyObject();
 
   bool inherits( int type ) const;
-  bool isInternal() const;
 
   const ObjectImp* imp() const;
   Objects parents() const;
 
   const Object* parent() const;
+  Object* parent();
   int propId() const;
 
   void draw( KigPainter& p, bool showSelection ) const;
@@ -248,9 +257,32 @@ public:
 
   bool shown() const;
 
-  void delParent( Object* o );
+  const ObjectImpType* impRequirement( Object* o, const Objects& os ) const;
+};
 
-  int impRequirement( Object* o, const Objects& os ) const;
+/**
+ * This class is not an object.  It only serves to hold a reference to
+ * a couple of objects, so they don't get deleted..  you can
+ * manipulate the referneced objects using the ObjectWithParents API.
+ */
+class ReferenceObject
+  : public ObjectWithParents
+{
+public:
+  ReferenceObject();
+  ReferenceObject( Object* o );
+  ReferenceObject( const Objects& os );
+  ~ReferenceObject();
+
+  void clearParents();
+
+private:
+  const ObjectImp* imp() const;
+  void draw( KigPainter&, bool ) const;
+  bool inRect( const Rect&, const KigWidget& ) const;
+  bool shown() const;
+  void calc( const Args&, const KigDocument& );
+  bool contains( const Coordinate&, const KigWidget& ) const;
 };
 
 #endif
