@@ -27,14 +27,14 @@
 
 #include <cmath>
 
-static const ArgParser::spec argsspecav[] =
+static const ArgParser::spec argsspecTranslation[] =
 {
-  { ObjectImp::ID_VectorImp, "Translate by this vector" },
-  { ObjectImp::ID_AnyImp, "Translate this object" }
+  { VectorImp::stype(), I18N_NOOP("Translate by this vector") },
+  { ObjectImp::stype(), I18N_NOOP("Translate this object") }
 };
 
 TranslatedType::TranslatedType()
-  : ArgparserObjectType( "Translation", argsspecav, 2 )
+  : ArgparserObjectType( "Translation", argsspecTranslation, 2 )
 {
 }
 
@@ -53,21 +53,21 @@ ObjectImp* TranslatedType::calc( const Args& targs, const KigDocument& ) const
   if ( targs.size() != 2 ) return new InvalidImp;
   Args args = margsparser.parse( targs );
   if ( !args[0] || ! args[1] ) return new InvalidImp;
-  assert( args[0]->inherits( ObjectImp::ID_VectorImp ) );
+  assert( args[0]->inherits( VectorImp::stype() ) );
 
   Coordinate dir = static_cast<const VectorImp*>( args[0] )->dir();
   Transformation t = Transformation::translation( dir );
   return args[1]->transform( t );
 }
 
-static const ArgParser::spec argsspecap[] =
+static const ArgParser::spec argsspecPointReflection[] =
 {
-  { ObjectImp::ID_PointImp, I18N_NOOP( "Reflect around this point" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Reflect this object" ) }
+  { PointImp::stype(), I18N_NOOP( "Reflect around this point" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Reflect this object" ) }
 };
 
 PointReflectionType::PointReflectionType()
-  : ArgparserObjectType( "PointReflection", argsspecap, 2 )
+  : ArgparserObjectType( "PointReflection", argsspecPointReflection, 2 )
 {
 }
 
@@ -87,19 +87,20 @@ ObjectImp* PointReflectionType::calc( const Args& targs, const KigDocument& ) co
   Args args = margsparser.parse( targs );
   if( !args[0] ) return new InvalidImp;
 
+  assert( args[0]->inherits( PointImp::stype() ) );
   Coordinate center = static_cast<const PointImp*>( args[0] )->coordinate();
   Transformation t = Transformation::pointReflection( center );
   return args[1]->transform( t );
 }
 
-static const ArgParser::spec argsspecal[] =
+static const ArgParser::spec argsspecLineReflection[] =
 {
-  { ObjectImp::ID_LineImp, I18N_NOOP( "Reflect over this line" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Reflect this object" ) }
+  { AbstractLineImp::stype(), I18N_NOOP( "Reflect over this line" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Reflect this object" ) }
 };
 
 LineReflectionType::LineReflectionType()
-  : ArgparserObjectType( "LineReflection", argsspecal, 2 )
+  : ArgparserObjectType( "LineReflection", argsspecLineReflection, 2 )
 {
 }
 
@@ -124,15 +125,15 @@ ObjectImp* LineReflectionType::calc( const Args& targs, const KigDocument& ) con
   return args[1]->transform( t );
 }
 
-static const ArgParser::spec argsspecpa[] =
+static const ArgParser::spec argsspecRotation[] =
 {
-  { ObjectImp::ID_PointImp, I18N_NOOP( "Rotate around this point" ) },
-  { ObjectImp::ID_AngleImp, I18N_NOOP( "Rotate by this angle" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Rotate this object" ) }
+  { PointImp::stype(), I18N_NOOP( "Rotate around this point" ) },
+  { AngleImp::stype(), I18N_NOOP( "Rotate by this angle" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Rotate this object" ) }
 };
 
 RotationType::RotationType()
-  : ArgparserObjectType( "Rotation", argsspecpa, 3 )
+  : ArgparserObjectType( "Rotation", argsspecRotation, 3 )
 {
 }
 
@@ -158,15 +159,15 @@ ObjectImp* RotationType::calc( const Args& targs, const KigDocument& ) const
   return args[2]->transform( Transformation::rotation( angle, center ) );
 }
 
-static const ArgParser::spec argsspecps[] =
+static const ArgParser::spec argsspecScalingOverCenter[] =
 {
-  { ObjectImp::ID_PointImp, I18N_NOOP( "Scale with this center" ) },
-  { ObjectImp::ID_SegmentImp, I18N_NOOP( "Scale by the length of this segment" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Scale this object" ) }
+  { PointImp::stype(), I18N_NOOP( "Scale with this center" ) },
+  { SegmentImp::stype(), I18N_NOOP( "Scale by the length of this segment" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Scale this object" ) }
 };
 
 ScalingOverCenterType::ScalingOverCenterType()
-  : ArgparserObjectType( "ScalingOverCenter", argsspecps, 3 )
+  : ArgparserObjectType( "ScalingOverCenter", argsspecScalingOverCenter, 3 )
 {
 }
 
@@ -192,15 +193,15 @@ ObjectImp* ScalingOverCenterType::calc( const Args& targs, const KigDocument& )
   return args[2]->transform( Transformation::scaling( ratio, center ) );
 }
 
-static const ArgParser::spec argsspecls[] =
+static const ArgParser::spec argsspecScalingOverLine[] =
 {
-  { ObjectImp::ID_LineImp, I18N_NOOP( "Scale over this line" ) },
-  { ObjectImp::ID_SegmentImp, I18N_NOOP( "Scale by the length of this segment" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Scale this object" ) }
+  { SegmentImp::stype(), I18N_NOOP( "Scale by the length of this segment" ) },
+  { AbstractLineImp::stype(), I18N_NOOP( "Scale over this line" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Scale this object" ) }
 };
 
 ScalingOverLineType::ScalingOverLineType()
-  : ArgparserObjectType( "ScalingOverLine", argsspecls, 3 )
+  : ArgparserObjectType( "ScalingOverLine", argsspecScalingOverLine, 3 )
 {
 }
 
@@ -220,21 +221,21 @@ ObjectImp* ScalingOverLineType::calc( const Args& targs, const KigDocument& ) co
   Args args = margsparser.parse( targs );
   if( !args[0] || ! args[1] ) return new InvalidImp;
 
-  LineData line = static_cast<const AbstractLineImp*>( args[0] )->data();
-  double ratio = static_cast<const SegmentImp*>( args[1] )->length();
+  double ratio = static_cast<const SegmentImp*>( args[0] )->length();
+  LineData line = static_cast<const AbstractLineImp*>( args[1] )->data();
 
   return args[2]->transform( Transformation::scaling( ratio, line ) );
 }
 
-static const ArgParser::spec argsspecra[] =
+static const ArgParser::spec argsspecProjectiveRotation[] =
 {
-  { ObjectImp::ID_RayImp, I18N_NOOP( "Projectively rotate with this ray" ) },
-  { ObjectImp::ID_AngleImp, I18N_NOOP( "Projectively rotate by this angle" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Projectively rotate this object" ) }
+  { RayImp::stype(), I18N_NOOP( "Projectively rotate with this ray" ) },
+  { AngleImp::stype(), I18N_NOOP( "Projectively rotate by this angle" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Projectively rotate this object" ) }
 };
 
 ProjectiveRotationType::ProjectiveRotationType()
-  : ArgparserObjectType( "ProjectiveRotation", argsspecra, 3 )
+  : ArgparserObjectType( "ProjectiveRotation", argsspecProjectiveRotation, 3 )
 {
 }
 
@@ -253,29 +254,29 @@ ObjectImp* ProjectiveRotationType::calc( const Args& targs, const KigDocument& )
   if ( targs.size() < 2 ) return new InvalidImp;
   Args args = margsparser.parse( targs );
   if ( !args[2] || !args[0] ) return new InvalidImp;
-  assert( args[0]->inherits( ObjectImp::ID_RayImp ) );
+  assert( args[0]->inherits( RayImp::stype() ) );
   const RayImp* ray = static_cast<const RayImp*>( args[0] );
   Coordinate c1 = ray->data().a;
   Coordinate dir = ray->data().dir().normalize();
   double alpha = 0.1*M_PI/2;
   if ( args[1] )
   {
-    assert( args[1]->inherits( ObjectImp::ID_AngleImp ) );
+    assert( args[1]->inherits( AngleImp::stype() ) );
     alpha = static_cast<const AngleImp*>( args[1] )->size();
   };
   return args[2]->transform(
     Transformation::projectiveRotation( alpha, dir, c1 ) );
 }
 
-static const ArgParser::spec argsspecpl[] =
+static const ArgParser::spec argsspecCastShadow[] =
 {
-  { ObjectImp::ID_PointImp, I18N_NOOP( "Cast a shadow from this light source" ) },
-  { ObjectImp::ID_LineImp, I18N_NOOP( "Cast a shadow on the plane defined by this line" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Cast the shadow of this object" ) }
+  { PointImp::stype(), I18N_NOOP( "Cast a shadow from this light source" ) },
+  { AbstractLineImp::stype(), I18N_NOOP( "Cast a shadow on the plane defined by this line" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Cast the shadow of this object" ) }
 };
 
 CastShadowType::CastShadowType()
-  : ArgparserObjectType( "CastShadow", argsspecpl, 3 )
+  : ArgparserObjectType( "CastShadow", argsspecCastShadow, 3 )
 {
 }
 
@@ -300,44 +301,44 @@ ObjectImp* CastShadowType::calc( const Args& targs, const KigDocument& ) const
     Transformation::castShadow( lightsrc, d ) );
 }
 
-int TranslatedType::resultId() const
+const ObjectImpType* TranslatedType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int PointReflectionType::resultId() const
+const ObjectImpType* PointReflectionType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int LineReflectionType::resultId() const
+const ObjectImpType* LineReflectionType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int RotationType::resultId() const
+const ObjectImpType* RotationType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int ScalingOverCenterType::resultId() const
+const ObjectImpType* ScalingOverCenterType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int ScalingOverLineType::resultId() const
+const ObjectImpType* ScalingOverLineType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int ProjectiveRotationType::resultId() const
+const ObjectImpType* ProjectiveRotationType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
-int CastShadowType::resultId() const
+const ObjectImpType* CastShadowType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
 bool TranslatedType::isTransform() const
@@ -380,14 +381,14 @@ bool CastShadowType::isTransform() const
   return true;
 }
 
-static const ArgParser::spec argsspecatot[] =
+static const ArgParser::spec argsspecApplyTransformation[] =
 {
-  { ObjectImp::ID_TransformationImp, I18N_NOOP( "Transform using this transformation" ) },
-  { ObjectImp::ID_AnyImp, I18N_NOOP( "Transform this object" ) }
+  { TransformationImp::stype(), I18N_NOOP( "Transform using this transformation" ) },
+  { ObjectImp::stype(), I18N_NOOP( "Transform this object" ) }
 };
 
 ApplyTransformationObjectType::ApplyTransformationObjectType()
-  : ArgparserObjectType( "ApplyTransformation", argsspecatot, 2 )
+  : ArgparserObjectType( "ApplyTransformation", argsspecApplyTransformation, 2 )
 {
 }
 
@@ -406,13 +407,13 @@ ObjectImp* ApplyTransformationObjectType::calc( const Args& targs, const KigDocu
   assert( targs.size() == 2 );
   const Args& args = margsparser.parse( targs );
   assert( args[0] && args[1] );
-  assert( args[0]->inherits( ObjectImp::ID_TransformationImp ) );
+  assert( args[0]->inherits( TransformationImp::stype() ) );
   return args[1]->transform( static_cast<const TransformationImp*>( args[0] )->data() );
 }
 
-int ApplyTransformationObjectType::resultId() const
+const ObjectImpType* ApplyTransformationObjectType::resultId() const
 {
-  return ObjectImp::ID_AnyImp;
+  return ObjectImp::stype();
 }
 
 bool ApplyTransformationObjectType::isTransform() const
